unit Helios_Basic;
{
Description: Used to emulate basic helios things.
}

interface

Uses Geometry,Windows,Graphics,Math3d,OpenGl15,Helios_Timer, Helios_VertexFragmentProgram,Helios_Basic_Types,Helios_Shader_System;

//Helios_Global.Pas
Const
HELIOS_MODFOLDER = 'Main';
DEPTH_OF_VIEW = 4000;
FOV = 45;

Var
GAME_FOLDER,MOD_FOLDER,BaseDir : String;
SCREEN_WIDTH,SCREEN_HEIGHT : Integer;
lastError : Cardinal;
lastErrorString : String;
h_DC   : HDC;
h_RC   : HGLRC;
h_Wnd  : HWND;
UseCameraByMouse : Boolean; // Not Used
EnvironmentPos : TVector3f;
FakeLightmap : TVector3f;
FakeLightmapB : Boolean;
LightPos : TVector3f;
UseBumpMaps : Boolean = True;
LastTexID : Integer;
ModelHeight : Single = -40;

ShaderManager : TShaderManager;

type
TRGBA = array[0..3] of GLUbyte;

//Q3BSP.Pas
var fogBuffer : array of single;
    inputBuffer : array[0..16383] of TBSPVertex;
    texCoordBuffer :  array[0..16383] of TVector2f;
    colorBuffer : array[0..16383] of TRGBA;

//Helios_Global.Pas
function StrToFloat(s : string) : extended;
Function FindStart(S : string) : String;
Function CleanStr(S : string) : string;
Procedure GetFirstBit(Const S : string; Var First,Rest : String);
Function TColorToTVector3f(Color : TColor) : TVector3f;
Function TVector3fToTColor(Vector3f : TVector3f) : TColor;

//Helios_Engine.Pas
Procedure ChangeModFolder(ModName : string);
Procedure InitiateHelios(GameFolder,ModFolder, Params : String);
procedure glResizeWnd(Width, Height : Integer);
Procedure DrawModelGrid;

//Helios_FileSystem.Pas
Function DoesFileExist(Filename : String) : String;

//Q3BSP.Pas
procedure SetDefaults;

//Helios_Basic
Function GetCurrentPosition : TVector3f;

implementation

Uses SysUtils;

//Helios_Global.Pas
function StrToFloat(s : string) : extended;
begin
  try
   // s := StringReplace(s, '.', ',', [rfReplaceAll]);
    result := sysutils.StrToFloat(s);
  except
  Try
    s := StringReplace(s, '.', ',', [rfReplaceAll]);
    result := sysutils.StrToFloat(s);
  except
    result := 0.0;
  end;
  end;
end;

Function FindStart(S : string) : String;
var
x,y : integer;
begin

y := -1;
for x := 1 to length(S) do
if (S[x] <> ' ') and (S[x] <> #9) then
begin
y := x;
Break;
end;

if y = -1 then
begin
Result := S;
exit;
end;

Result := Copy(s,y,length(S));

end;

Function CleanStr(S : string) : string;
var
x : integer;
begin
Result := '';

for x := 1 to length(S) do
if S[x] <> '"' then
Result := Result + S[x];

end;

Procedure GetFirstBit(Const S : string; Var First,Rest : String);
Var
Str : String;
x,y : integer;
B : Boolean;
begin
Str := S;
Str := FindStart(Str);


B := false;
y := -1;
for x := 1 to length(Str) do
if (S[x] = '"') then
B := not B
else
if (S[x] = ' ') and not B then
Begin
y := x;
Break;
end;

if y = -1 then
begin
First := Str;
Rest := '';
Exit;
end;

First := Copy(Str,1,y-1);
Rest := Copy(Str,y,length(Str));

end;

Function TColorToTVector3f(Color : TColor) : TVector3f;
begin

Result.X := GetRValue(Color) / 255;
Result.Y := GetGValue(Color) / 255;
Result.Z := GetBValue(Color) / 255;

end;

Function TVector3fToTColor(Vector3f : TVector3f) : TColor;
begin

Result := RGB(trunc(Vector3f.X*255),trunc(Vector3f.Y*255),trunc(Vector3f.Z*255));

end;

//Helios_Engine.Pas
Function DoesModFolderExist(ModName : string) : Boolean;
begin
Result := fileexists(GAME_FOLDER + '\' + ModName + '\liblist.gam');
end;


Procedure ChangeModFolder(ModName : string);
begin
  if DoesModFolderExist(ModName) then
  MOD_FOLDER := ModName;

  BumpVP := ARB_LoadShader(GL_VERTEX_PROGRAM_ARB, DoesFileExist('\shader\bump_vp.txt'));
  BumpFP := ARB_LoadShader(GL_FRAGMENT_PROGRAM_ARB, DoesFileExist('\shader\bump_fp.txt'));
  ShaderManager.Destroy;
  ShaderManager := TShaderManager.Create(GAME_FOLDER+'\'+HELIOS_MODFOLDER);
end;

Procedure InitiateHelios(GameFolder,ModFolder, Params : String);
begin
GAME_FOLDER := GameFolder;
MOD_FOLDER := ModFolder;
gTimer := THiResTimer.Create;

BumpVP := ARB_LoadShader(GL_VERTEX_PROGRAM_ARB, DoesFileExist('\shader\bump_vp.txt'));
BumpFP := ARB_LoadShader(GL_FRAGMENT_PROGRAM_ARB, DoesFileExist('\shader\bump_fp.txt'));

ShaderManager := TShaderManager.Create(GAME_FOLDER+'\'+HELIOS_MODFOLDER);

glEnable(GL_CULL_FACE);
glCullFace(GL_FRONT);
glEnable(GL_DEPTH_TEST);
glDepthFunc(GL_LEQUAL);
end;

procedure glResizeWnd(Width, Height : Integer);
begin
SCREEN_WIDTH := Width;
SCREEN_HEIGHT := Height;

  if (Height = 0) then                // prevent divide by zero exception
    Height := 1;
  glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
  glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
  glLoadIdentity();                   // Reset View
//  gluPerspective(FOV, Width/Height, 1.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
  gluPerspective(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
//  PerspectiveGL(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);
//   glFrustum(-40.0, 40.0, -20.704225, 20.704225, 4.0, 4096.0);  //GLQuake Mode
  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
  glLoadIdentity();                   // Reset View
end;

//Helios_FileSystem.Pas
Function DoesFileExist(Filename : String) : String; // If Exists filename is given as result
begin

if MOD_FOLDER <> '' then
if fileexists(GAME_FOLDER+'\'+MOD_FOLDER+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+MOD_FOLDER+'\'+Filename;
Exit;
end;

if fileexists(GAME_FOLDER+'\'+HELIOS_MODFOLDER+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+HELIOS_MODFOLDER+'\'+Filename;
Exit;
end;

If Basedir <> '' then
if fileexists(GAME_FOLDER+'\'+Basedir+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+Basedir+'\'+Filename;
Exit;
end;

if fileexists(Filename) then
begin
Result := Filename;
Exit;
end;

Result := '';

end;

//Q3BSP.Pas
procedure SetDefaults;
begin
  //glDepthFunc(GL_LEQUAL);  //
  //glEnable(GL_DEPTH_TEST); //
  glDisable(GL_ALPHA_TEST);
  glDepthMask(true);
  glDisable(GL_BLEND);
  {glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);}

  glDisable(GL_COLOR);  //
{  if CG_MultiTexture then begin //
    glActiveTexture(GL_TEXTURE1);
    glDisableClientState(GL_COLOR_ARRAY);
    glActiveTexture(GL_TEXTURE0);
  end;   }
  glDisableClientState(GL_COLOR_ARRAY);   //
  glColor3ub(255,255,255);  //
  glCullFace(GL_FRONT);
  //glEnable(GL_CULL_FACE);  //
end;

Function GetCurrentPosition : TVector3f;
Var
Matrix : TMatrix;
begin

glGetFloatv(GL_MODELVIEW_MATRIX, @Matrix);

  Result.X := Matrix[3,0];
  Result.Y := Matrix[3,1];
  Result.Z := Matrix[3,2];

end;

// "Ripped" From Helios_Engine.pas
Procedure DrawGridSquare(x,y,z : integer; Colour : TVector3f; Size : Single);
var
Sx,Sy,Sz : single;
begin

Sx := Size*x - Size / 2;
Sy := y;
Sz := Size*z - Size / 2;

glDisable(GL_CULL_FACE);
glEnable(GL_TEXTURE_2D);

glBindTexture(GL_TEXTURE_2D,0);

glColor3f(Colour.X,Colour.Y,Colour.Z);

glBegin( GL_LINES );

glVertex3f(Sx+0, Sy+0, Sz+0);
glVertex3f(Sx+Size, Sy+0, Sz+0);

glVertex3f(Sx+Size, Sy+0, Sz+0);
glVertex3f(Sx+Size, Sy+0, Sz+Size);

glVertex3f(Sx+Size, Sy+0, Sz+Size);
glVertex3f(Sx+0, Sy+0, Sz+Size);

glVertex3f(Sx+0, Sy+0, Sz+Size);
glVertex3f(Sx+0, Sy+0, Sz+0);

glEnd();

end;

Procedure DrawModelGrid;
var
x,z : integer;
begin

for x := -8 to 8 do
for z := -8 to 8 do
DrawGridSquare(x,0,z,SetVector(0.3,0.3,0.3),8);

end;

begin

FakeLightmap := TColorToTVector3f(RGB(255,255,255));
FakeLightmapB := False;

end.
